#ifndef MENUSTORAGE_H_CONTEXTMENU_HEADER
#define MENUSTORAGE_H_CONTEXTMENU_HEADER
// Nathan E. Moore
// March 8, 2005

#include "windows_include.h"
#include <loki/SmartPtr.h>

namespace utility
{
	template <typename T>
	class MenuStorage
	{
	private:
		struct NoType {};
	protected:
		typedef T* StoredType;
		typedef T* PointerType;
		typedef NoType& ReferenceType;

	public:
		MenuStorage(const StoredType& hmenu)
			:hmenu_(hmenu)
		{}

        void Swap(MenuStorage& rhs)
        {
			std::swap(hmenu_, rhs.hmenu_);
		}

		friend inline PointerType GetImpl(const MenuStorage& ms)
		{
			return ms.hmenu_;
		}

		friend inline const StoredType& GetImplRef(const MenuStorage& ms)
		{
			return ms.hmenu_;
		}

		friend inline StoredType& GetImplRef(MenuStorage& ms)
		{
			return ms.hmenu_;
		}

	protected:
		~MenuStorage()
		{}

		void Destroy()
		{
			if (hmenu_)
				DestroyMenu(hmenu_);
		}

		static StoredType Default()
		{
			return NULL;
		}

	private:
		StoredType hmenu_;
	};

	namespace smartmenu_impl
	{
#ifdef STRICT
		typedef HMENU__ HmenuType;
#else
		typedef void HmenuType;
#endif
	}

	typedef Loki::SmartPtr<smartmenu_impl::HmenuType, Loki::RefCounted, Loki::AllowConversion, Loki::AssertCheck, MenuStorage> SmartMenu;
}	// end namespace utility

#endif // MENUSTORAGE_H_CONTEXTMENU_HEADER
