#ifndef STORAGE_H_CONTEXTMENU_HEADER
#define STORAGE_H_CONTEXTMENU_HEADER
// Nathan E. Moore
// March 5, 2005

#include "common.h"
#include <cstddef>  // for std::ptrdiff_t
#include <cstdlib>  // for std::free

namespace utility
{
	// Note that for a smart pointer using this storage implemantation
	// if x is a smart pointer, the expressions:
	//     *x
	//     x->something()
	// are not valid. Instead a indexed access is provided. ala
	//     x[1];
	template <typename T>
	class ArrayStorage
	{
	protected:
		typedef T* StoredType;
		typedef T* PointerType;
		typedef T& ReferenceType;

	public:

		ArrayStorage()
			:p_(Default())
		{}

		ArrayStorage(const StoredType& ptr)
			:p_(ptr)
		{}

		//PointerType operator->() const
		//{
		//	return p_;
		//}

		//ReferenceType operator*() const
		//{
		//	return *p_;
		//}

		ReferenceType operator[](std::ptrdiff_t i) const
		{
			ASSERTION(p_ != 0);
			ASSERTION(i >= 0);

			return p_[i];
		}

        void Swap(ArrayStorage& rhs)
        {
			std::swap(p_, rhs.p_);
		}

		friend inline PointerType GetImpl(const ArrayStorage& as)
		{
			return as.p_;
		}

		friend inline const StoredType& GetImplRef(const ArrayStorage& as)
		{
			return as.p_;
		}

		friend inline StoredType& GetImplRef(ArrayStorage& as)
		{
			return as.p_;
		}

	protected:
		~ArrayStorage()
		{}

		void Destroy()
		{
			delete [] p_;
		}

		static StoredType Default()
		{
			return NULL;
		}

	private:
		StoredType p_;
	};	// end class ArrayStorage

	template <typename T>
	class MallocStorage
	{
	protected:
		typedef T* StoredType;
		typedef T* PointerType;
		typedef T& ReferenceType;

	public:

		MallocStorage()
			:p_(Default())
		{}

		MallocStorage(const StoredType& ptr)
			:p_(ptr)
		{}

		PointerType operator->() const
		{
			return p_;
		}

		ReferenceType operator*() const
		{
			return *p_;
		}

		friend inline PointerType GetImpl(const MallocStorage& ms)
		{
			return ms.p_;
		}

		friend inline const StoredType& GetImplRef(const MallocStorage& ms)
		{
			return ms.p_;
		}

		friend inline StoredType& GetImplRef(MallocStorage& ms)
		{
			return ms.p_;
		}

	protected:
		~MallocStorage()
		{}

		void Destroy()
		{
			std::free(p_);
		}

		static StoredType Default()
		{
			return NULL;
		}

	private:
		StoredType p_;
	}; // end class MallocStorage
}	// end namespace utility

#endif // STORAGE_H_CONTEXTMENU_HEADER