#ifndef TASKALLOC_H_CONTEXTMENU_HEADER
#define TASKALLOC_H_CONTEXTMENU_HEADER
// Nathan E. Moore
// March 2, 2005

#include <objidl.h>
#include <loki/SmartPtr.h>
#include <loki/Singleton.h>

namespace com
{
	// This provides a single IMalloc instance for the entire application to use.
	// Note that it is not thread safe. 
	class TaskAlloc
	{
	public:
		typedef Loki::SmartPtr<IMalloc, Loki::COMRefCounted> IMallocPtr;
		static void* Alloc(std::size_t);
		static void Free(void*);
		static IMallocPtr& Get();

	private:
		typedef Loki::SingletonHolder<TaskAlloc> TASingleton;
		friend struct Loki::CreateUsingNew<TaskAlloc>;

		IMallocPtr allocator_;

		TaskAlloc();

		// no copies
		TaskAlloc(const TaskAlloc&);
		TaskAlloc& operator=(const TaskAlloc&);
	};	// end class TaskAllocator

	// Stoage policy for Loki::SmartPtr, uses TaskAlloc::Free
	// to delete the stored pointer
	template <typename T>
	class TaskAllocStorage
	{
	protected:
		typedef T* StoredType;
		typedef T* PointerType;
		typedef T& ReferenceType;

	public:

		TaskAllocStorage()
			:p_(Default())
		{}

		TaskAllocStorage(const StoredType& ptr)
			:p_(ptr)
		{}

		PointerType operator->() const
		{
			return p_;
		}

		ReferenceType operator*() const
		{
			return *p_;
		}

		friend inline PointerType GetImpl(const TaskAllocStorage& tas)
		{
			return tas.p_;
		}

		friend inline const StoredType& GetImplRef(const TaskAllocStorage& tas)
		{
			return tas.p_;
		}

		friend inline StoredType& GetImplRef(TaskAllocStorage& tas)
		{
			return tas.p_;
		}

	protected:
		~TaskAllocStorage()
		{}

		void Destroy()
		{
			TaskAlloc::Free(p_);
		}

		static StoredType Default()
		{
			return NULL;
		}

	private:
		StoredType p_;
	};	// end class TaskAllocStorage
}	// end namespace com

#endif // end TASKALLOC_H_CONTEXTMENU_HEADER
